import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.math.*;
import java.awt.image.*;

/**This class draws the images of the big Voronoi decomposition
under the action of the symmetry groupoid of the
Octahedral plan*/



public class DrawVoronoiImages {


    public static void main(PictureCanvas P,Graphics2D g) {
	main1(P,g);
	main2(P,g);
    }
    
    public static  void main1(PictureCanvas P,Graphics2D g) {
	for(int q=0;q<6;++q) {
	for(int k=0;k<6;++k) {
	  Complex[] Z=Voronoi.candidates(P.SOURCE,k);
	  Z=Symmetry.symm(q,Z);
	  drawPoly(P,g,Z,ColorCanvas.C[1][k]);
	}}
    }

    public static void main2(PictureCanvas P,Graphics2D g) {
	int[][] A=specialPairs();
	    for(int i=0;i<A.length;++i) {
		int q=A[i][0];
		int k=A[i][1];
	        Complex[] Z=Voronoi.candidates(P.SOURCE,k);
	        Z=Symmetry.symm(q,Z);
		Z=augment(q,Z);
		drawPoly(P,g,Z,ColorCanvas.C[1][k]);
	    }
    }


    public static Complex specialPoint(int q) {
	Complex z0=new Complex(1,0);
	Complex z1=new Complex(-.5,Math.sqrt(3)/2);
	Complex z2=new Complex(-.5,-Math.sqrt(3)/2);
	Complex[] Z={z0,z1,z1,z2,z2,z0};
	return Z[q];
    }


    public static Complex[] augment(int q,Complex[] Z) {
	Complex[] W=new Complex[Z.length+1];
	for(int i=0;i<Z.length;++i) W[i]=new Complex(Z[i]);
	W[Z.length]=specialPoint(q);
	return W;
    }

    
    public static int[][] specialPairs() {
	int[][] A={{0,0},{0,1},{5,0},{5,1},{4,2},{4,3},{3,2},{3,3},{2,4},{2,5},{1,4},{1,5}};
	return A;
    }


    public static void drawPoly(PictureCanvas P,Graphics2D g,Complex[] Z,Color C) {
        PolygonWrapper X=new PolygonWrapper(Z.length,Z);
	try{X=PolygonWrapper.convexHull(X);}
	catch(Exception e) {}
	Path2D.Double gp=X.toPath();
	gp=P.transform(gp);
	g.setColor(C);
	g.fill(gp);
    }
    

}


