import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class Octahedron {


    public static Complex cr(int j) {
        double c=Math.cos(2*j*Math.PI/3);
	double s=Math.sin(2*j*Math.PI/3);
	Complex u=new Complex(c,s);
	return(u);
    }

    /**This is the key routine.  Start with the point z0,
       take the antipode point, then develop it out to the
       6 points.   These 6 points should be the image of
       z0 under glide reflections.*/

    public static Complex[] symmetrize(Complex z0) {
	Complex[] w=new Complex[6]; 
	Complex z1=anti(z0);
	z1=Complex.times(cr(2),Complex.conjugate(z1));
	z1=anti(z1);
	for(int i=0;i<6;++i) w[i]=specialMap(i,z1);
	return(w);
    }

    public static Path2D.Double shortestPath(Complex z0,Complex z) {
	Complex w0=anti(z);
	Complex[] w=symmetrize(z);
	int i=shortIndex(z0,z);
	Path2D.Double gp=new Path2D.Double();
	double x0=(w0.x);
	double y0=(w0.y);
	double x=(w[i].x);
	double y=(w[i].y);
	gp.moveTo(x,y);
	gp.lineTo(x0,y0);
	gp.closePath();
	return(gp);
    }

    public static int shortIndex(Complex z0,Complex z) {
	Complex w0=anti(z);
	Complex[] w=symmetrize(z);
	int index=-1;
	double min=100;
	for(int i=0;i<6;++i) {
	    double test=Complex.dist(z0,w[i]);
	    if(test<min) {
		min=test;
		index=i;
	    }
	}
	return(index);
    }

    public static boolean isShort(Complex z0,Complex z,int k) {
	Complex w0=anti(z);
	Complex[] w=symmetrize(z);
	double min=100;
	for(int i=0;i<6;++i) {
	    double test=Complex.dist(z0,w[i]);
	    if(test<min) {
		min=test;
	    }
	}
	double test2=Complex.dist(z0,w[k]);
	if(test2<min+.000000001) return true;
	return(false);
    }



    public static Path2D.Double symmPath(Complex z) {
	Complex w0=anti(z);
	Complex[] w=symmetrize(z);
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<6;++i) {
	    double x0=(w0.x);
	    double y0=(w0.y);
	    double x=(w[i].x);
	    double y=(w[i].y);
	    gp.moveTo(x,y);
	    gp.lineTo(x0,y0);
	}
	gp.closePath();
	return(gp);
    }


    public static double symmDist(Complex z0,Complex z1) {
	Complex[] w1=symmetrize(z1);
	double min=100;
	for(int i=0;i<6;++i) {
	    double test=Complex.dist(z0,w1[i]);
	    if(min>test) min=test;
	}
	return(min);
    }



    public static Complex specialPoint(int k) {
	if(k==0) return(specialPoint0());
	if(k==1) return(specialPoint1());
	if(k==2) return(specialPoint2());
	if(k==3) return(specialPoint3());
	if(k==4) return(specialPoint4());
	if(k==5) return(specialPoint5());
	return(null);
    }


    /**The important isometies*/

    public static Complex specialPoint() {
         double c=Math.cos(Math.PI/3);
	 double s=Math.sin(Math.PI/3);
         Complex w=new Complex(c,3*s);
	 w=Complex.conjugate(w);
	 w=Complex.times(w,cr(1));
	 return(w);
    }

    public static Complex anti(Complex z) {
	Complex w=specialPoint();
	w=Complex.minus(w,z);
	return(w);
    }


    public static Complex specialMap(int k,Complex z) {
	Complex w=new Complex(z);
	Complex u=specialTwist(k);
	w=Complex.minus(z,specialPoint(0));
	w=Complex.times(w,u);
	w=Complex.plus(w,specialPoint(k));
	return(w);
    }

    public static Complex specialMapInverse(int k,Complex z) {
	Complex w=new Complex(z);
	Complex u=specialTwistInverse(k);
	w=Complex.minus(z,specialPoint(k));
	w=Complex.times(w,u);
	w=Complex.plus(w,specialPoint(0));
	return(w);
    }

    public static Complex specialTwist(int k) {
	if(k==0) return(cr(0));
	if(k==1) return(cr(2));
	if(k==2) return(cr(1));
	if(k==3) return(cr(0));
	if(k==4) return(cr(2));
	if(k==5) return(cr(1));
	return(null);
    }

    public static Complex specialTwistInverse(int k) {
	if(k==0) return(cr(0));
	if(k==1) return(cr(1));
	if(k==2) return(cr(2));
	if(k==3) return(cr(0));
	if(k==4) return(cr(1));
	if(k==5) return(cr(2));
	return(null);
    }


    public static Complex specialPoint0() {
	return(specialPoint());
    }

    public static Complex specialPoint2() {
	Complex z=specialPoint0();
	z=Complex.times(z,cr(2));
	return(z);
    }

    public static Complex specialPoint4() {
	Complex z=specialPoint0();
	z=Complex.times(z,cr(1));	
        return(z);
    }

    public static Complex specialPoint1() {
	Complex z=specialPoint0();
	return(Complex.conjugate(z));
    }


    public static Complex specialPoint5() {
	Complex z=specialPoint1();
	z=Complex.times(z,cr(1));
 	return(z);
    }

    public static Complex specialPoint3() {
	Complex z=specialPoint1();
	z=Complex.times(z,cr(2));	
        return(z);
    }



    public static Complex findRival(Complex z) {
	Complex Z=new Complex();
	Path2D.Double gp=Triangulation.tri(2,1,1);
	Complex sp=specialPoint();
	double t=.0001;
	Complex[] X=candidates(z,true);
	double max=0;
	for(int i=0;i<X.length;++i) {
	    double test=symmDist(z,X[i]);
	    Complex temp=new Complex(X[i]);
	    temp.x=(1-t)*temp.x+t*sp.x;
	    temp.y=(1-t)*temp.y+t*sp.y;
	    boolean cont=gp.contains(temp.x,temp.y);
	    if((cont==true)&&(max<test)) {
	       max=test;
	       Z=new Complex(X[i]);
	       Z.tag[0]=i;
	    }
	}
	return(Z);
    }

    public static Complex[] candidates(Complex z,boolean FIX) {
	Complex[][] W=bisectorList(z);
	int count=0;
	int n=W.length;
	Complex[] X=new Complex[n];
	for(int i=0;i<n;++i) {
	    int j=(i+3)%12;
	    X[count]=getIntersection(W[i],W[j]);
	    X[count].tag[0]=i;
	    X[count].tag[1]=j;
	    if(FIX==true) X[count]=fix(X[count]);
	    ++count;
	}
	return(X);
    }


    public static Complex[][] bisectorList(Complex z) {
	int count=0;
	Complex[] w=new Complex[6];
	for(int i=0;i<6;++i) w[i]=Octahedron.specialMapInverse(i,z);
	Complex[][] W=new Complex[15][2];
	for(int i0=0;i0<6;++i0) {
	    for(int j0=i0+1;j0<6;++j0) {
	       W[count]=bisectorPoints(w[i0],w[j0]);
	       ++count;
	    }
	}
	W=trimList(W);
	return(W);
    }

    public static Complex[][] trimList(Complex[][] W) {
	Complex[][] W1=new Complex[15][2];
	for(int i=0;i<15;++i) {
	    W1[i][0]=new Complex(W[i][0]);
	    W1[i][1]=new Complex(W[i][1]);
	}

	double s3=Math.sqrt(3);
	Complex z0=new Complex(0,0);
	W1[0][0]=new Complex(1,0);
	W1[0][1]=unitize(W1[0],3);

	W1[1][0]=new Complex(2.5,-s3/2);
	W1[1][1]=unitize(W1[1],2*s3);

	W1[3][0]=new Complex(-1.0/2,1.5*s3);
	W1[3][1]=unitize(W1[3],-2*s3);

	W1[4][0]=new Complex(1,s3);	
	W1[4][1]=unitize(W1[4],-s3);

	W1[5][0]=Complex.conjugate(W1[1][0]);
	W1[5][1]=unitize(W1[5],s3);

	W1[6][0]=new Complex(4,0);
	W1[6][1]=unitize(W1[6],2*s3);

	W1[8][0]=new Complex(-1.0/2,s3/2);
	W1[8][1]=unitize(W1[8],-2*s3);

	W1[9][0]=new Complex(4,s3);
	W1[9][1]=unitize(W1[9],3);

	W1[10][0]=new Complex(4,2*s3);
	W1[10][1]=unitize(W1[10],2*s3);

	W1[12][0]=new Complex(2.5,1.5*s3);
	W1[12][1]=unitize(W1[12],s3);

	W1[13][0]=new Complex(2.5,2.5*s3);
	W1[13][1]=unitize(W1[13],2*s3);

	W1[14][0]=new Complex(1,2*s3);
	W1[14][1]=unitize(W1[14],3);

	Complex[][] W2=new Complex[12][2];
	int count=0;
	for(int i=0;i<15;++i) {
	    boolean test=true;
	    if(i==2) test=false;
	    if(i==7) test=false;
	    if(i==11) test=false;
	    if(test==true) {
		W2[count][0]=new Complex(W1[i][0]);
		W2[count][1]=new Complex(W1[i][1]);
		++count;
	    }
	}

	Complex[][] W3=new Complex[12][2];
	int[] per={7,8,9,10,11,2,3,6,0,1,4,5};
	for(int i=0;i<12;++i) {
	    W3[i][0]=new Complex(W2[per[i]][0]);
	    W3[i][1]=new Complex(W2[per[i]][1]);
	}
	return(W3);
    }

    public static Complex unitize(Complex[] W,double r) {
	Complex z1=Complex.minus(W[1],W[0]);
	double d=z1.norm();
	z1.x=r*z1.x/d;
	z1.y=r*z1.y/d;
	return(Complex.plus(W[0],z1));
    }


    public static Complex fix(Complex z0) {
	Complex z1=Octahedron.anti(z0);
	Complex c1=Octahedron.cr(2);
	z1=Complex.times(c1,Complex.conjugate(z1));
	z1=Octahedron.anti(z1);
	z1.tag[0]=z0.tag[0];
	z1.tag[1]=z0.tag[1];
	return(z1);
    }

    public static Complex getIntersection(Complex[] W1,Complex[] W2) {
	Complex x=Vector.findCross2(W1[0],W1[1],W2[0],W2[1]);
	return(x);
    }

    public static Path2D.Double bisector(Complex z1,Complex z2) {
	Complex[] W=bisectorPoints(z1,z2);
	Complex w2=W[0];
	Complex w3=W[1];
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo((w2.x),(w2.y));
	gp.lineTo((w3.x),(w3.y));
	return(gp);
    }


    public static Complex[] bisectorPoints(Complex z1,Complex z2) {
	Complex w1=Complex.plus(z1,z2);
	w1.x=w1.x/2;
	w1.y=w1.y/2;
	Complex w2=Complex.minus(z1,z2);
	Complex w3=Complex.minus(z1,z2);
	w2=Complex.times(w2,new Complex(0,10));
	w3=Complex.times(w3,new Complex(0,-10));
	w2=Complex.plus(w1,w2);
	w3=Complex.plus(w1,w3);
	Complex[] W={w2,w3};
	return(W);
    }





}


